pub struct FromBytesWithNulError { /* private fields */ }Expand description
A possible error value when converting a Vec<u8> into the requested string.
This acts as a wrapper type for the actual error, and an owned buffer that can be return the provided allocation.
Implementations
Sourceimpl FromBytesWithNulError
impl FromBytesWithNulError
Sourcepub fn kind(&self) -> NGCesu8CError
pub fn kind(&self) -> NGCesu8CError
The specific kind of error encountered
Sourcepub fn into_inner(self) -> Vec<u8> ⓘ
pub fn into_inner(self) -> Vec<u8> ⓘ
Receive the unmodified, originally provided owned byte string